/**
 * @Project: merchantClient
 * @Title: ConvertDataTypeUtil.java
 * @Package com.util
 * @Description: TODO(javeBean和Map互转工具类)
 * @author qiuqin10664 qiuqin10664@hundsun.com
 * @date 2015-1-22 
 * @Copyright: 2015 www.hundsun.com All Rights Reserved.
 * @version V1.0.0.0 
 */

package com.util;

import java.beans.IntrospectionException;
import java.beans.PropertyDescriptor;
import java.lang.reflect.Field;
import java.lang.reflect.InvocationTargetException;
import java.util.HashMap;
import java.util.Map;

public class ConvertDataTypeUtil {

	// 把一个javaBean实体类转换成一个Map（通过反射机制实现）
	public static Map<String, Object> convertBean2Map(Object obj) {
		if (obj == null) {
			throw new IllegalArgumentException("要转换的对象参数不能为空");
		}
		return saveToMap(obj.getClass(), obj);
	}

	private static Map<String, Object> saveToMap(Class<?> clazz, Object obj) {
		Field[] fields = clazz.getDeclaredFields();
		Map<String, Object> result = new HashMap<String, Object>();
		if (fields != null) {
			try {
				for (Field field : fields) {
					field.setAccessible(true); // 设置属性可以访问
					String propertyName = field.getName();
					PropertyDescriptor pd = new PropertyDescriptor(
							propertyName, clazz);
					Object value = pd.getReadMethod().invoke(obj);
					result.put(propertyName, value);
				}
			} catch (IntrospectionException e) {
				e.printStackTrace();
			} catch (IllegalAccessException e) {
				e.printStackTrace();
			} catch (IllegalArgumentException e) {
				e.printStackTrace();
			} catch (InvocationTargetException e) {
				e.printStackTrace();
			}
		}
		if (clazz.getSuperclass() != Object.class) {
			result.putAll(saveToMap(clazz.getSuperclass(), obj));
		}
		return result;
	}

	// 把一个Map转换成一个实体类JavaBean（通过反射机制实现）
	public static Object convertMap2Bean(Map<String, Object> map, Object object) {
		if (map == null || map.size() == 0) {
			return null;
		}
		if (object == null || "".equals(object)) {
			return null;
		}
		Class<?> clazz = null;
		clazz = object.getClass();
		// 获得对象的所有属性保存在map中
		getValueToBean(clazz, map, object);
		return object;
	}

	// 递归获取父类的属性并赋值
	public static void getValueToBean(Class<?> clazz, Map<String, Object> map,
			Object object) {
		if (clazz.getSuperclass() != null) {
			getValueToBean(clazz.getSuperclass(), map, object);
		}
		Field[] fields = clazz.getDeclaredFields();
		try {
			for (int i = 0; i < fields.length; i++) {
				Field field = fields[i];
				String propertyName = field.getName();
				if (map.containsKey(propertyName)) {
					field.setAccessible(true); // 设置属性可以访问
					new PropertyDescriptor(propertyName, clazz)
							.getWriteMethod().invoke(object,
									map.get(propertyName));
				}
			}
		} catch (IntrospectionException e) {
			e.printStackTrace();
		} catch (IllegalAccessException e) {
			e.printStackTrace();
		} catch (IllegalArgumentException e) {
			e.printStackTrace();
		} catch (InvocationTargetException e) {
			e.printStackTrace();
		}
	}
}
